body {
    font-family: Helvetica, Arial, sans-serif;
    font-size: 12px;
    min-width: 1200px;
    color: #000000;
}

h2 {
    font-size: 16px;
    color: black;
}

p {
    color: black;
}

a {
    color: #FF0000;
}

table {
    border-collapse: collapse;
}

/******************************
 * FINAL(COMBINED) REPORT
 ******************************/

.result_frame_title {
    font-size: 16px;
    font-weight: bold;
    padding: 10px;
}

.result_frame {
    border: none;
}

/******************************
 * SUMMARY INFORMATION
 ******************************/

#summary-wrapper {
    position: fixed;
    background-color: white;
    padding: 5px;
    border-color: rgb(200, 200, 200);
    border-style: solid;
    border-width: 3px;
}

#summary-space {
    height: 100px;
}

#configuration td {
    padding: 5px;
    border: 1px solid #E6E6E6;
}

#configuration tr:nth-child(odd) {
    background-color: #f6f6f6;
}

.clickable {
    cursor: pointer;
    text-decoration: underline;
}

/******************************
 * TEST RESULT COLORS
 ******************************/
span.passed, .passed .col-result {
    color: green;
}

span.skipped, span.xfailed, .skipped .col-result, .xfailed .col-result {
    color: orange;
}

span.error, span.failed, span.xpassed, .failed .col-result, .error .col-result, .xpassed .col-result {
    color: red;
}

.results-table-row:hover {
    background-color: rgb(240, 240, 240);
}

.failed.results-table-row {
    background-color: rgb(250, 230, 230);
}

.failed.results-table-row:hover {
    background-color: rgb(250, 180, 180);
}

.error.results-table-row {
    background-color: rgb(250, 200, 200);
}

.error.results-table-row:hover {
    background-color: rgb(250, 150, 150);
}

/******************************
 * RESULTS TABLE
 * 
 * 1. Table Layout
 * 2. Debug
 * 3. Sorting items
 * 
 ******************************/

/*------------------
 * 1. Table Layout
 *------------------*/

#results-table {
    border: 1px solid #e6e6e6;
    color: #000000;
    font-size: 12px;
    width: 100%
}

#results-table th, #results-table td {
    padding: 5px;
    border: 1px solid #E6E6E6;
    text-align: left
}

#results-table th {
    font-weight: bold
}

.results-table-row {
    cursor: pointer;
}
/*------------------
 * 2. Debug
 *------------------*/

.stacktrace:only-child {
    height: inherit
}

.stacktrace {
    background-color: #e6e6e6;
    border: 1px solid #e6e6e6;
    color: black;
    display: block;
    font-family: "Courier New", Courier, monospace;
    overflow-y: scroll;
    padding: 5px;
    white-space: pre-wrap;
}

.screenshot, .video {
    border: 1px solid #e6e6e6;
    float: right;
    height: 240px;
    margin-left: 5px;
    overflow: hidden;
    width: 320px
}

.screenshot img {
    width: 320px
}

/*if the result is passed or xpassed don't show debug row*/
.passed + .debug, .xpassed + .debug {
    display: none;
}

/*------------------
 * 3. Sorting items
 *------------------*/
.sortable {
    cursor: pointer;
}

.sort-icon {
    font-size: 0px;
    float: left;
    margin-right: 5px;
    margin-top: 5px;
    /*triangle*/
    width: 0;
    height: 0;
    border-left: 8px solid transparent;
    border-right: 8px solid transparent;
}

.inactive .sort-icon {
    /*finish triangle*/
    border-top: 8px solid #E6E6E6;
}

.asc.active .sort-icon {
    /*finish triangle*/
    border-bottom: 8px solid #999;
}

.desc.active .sort-icon {
    /*finish triangle*/
    border-top: 8px solid #999;
}
